import 'package:flutter/material.dart';

class Tile {
  Color color;
  String index;

  Tile(this.index,this.color);
}

class TileWidget extends StatelessWidget {
  Tile tile;
  bool isUsable=false;

  TileWidget(this.tile,this.isUsable);

  @override
  Widget build(BuildContext context) {
    return this.coloredBox();
  }

  Widget coloredBox() {
    return Container(
        color: tile.color,
        child: Align(
            child: Text(tile.index.toString(),),
            alignment: Alignment.center,
      )
    );
  }
}

BoxDecoration myBoxDecoration(bool isUsable) {
  BoxDecoration res=BoxDecoration();
  if(isUsable){
    res= BoxDecoration(
      border: Border.all(
        color: Colors.red,
        width: 5.0,
      ),
    );
  }
  return res;
}

List<TileWidget>createTileWidgets(double slide, int emptyIndex){
  List<TileWidget> tiles = <TileWidget>[];
  int max=slide.toInt()*slide.toInt();
  for(int i=0;i<max;i++){
    if(i==emptyIndex){
      tiles.add(TileWidget(Tile("Vide",Colors.white),false));
    }else if(i==emptyIndex+1 || i==emptyIndex+slide.toInt()){
      tiles.add(TileWidget(Tile((i).toString(), Colors.grey),true));
    }
    else tiles.add(TileWidget(Tile((i).toString(), Colors.grey),false));
  }
  return tiles;
}

void main() => runApp(new MaterialApp(home: PositionedTiles()));

class PositionedTiles extends StatefulWidget {
  @override
  State<StatefulWidget> createState() => PositionedTilesState();
}

class PositionedTilesState extends State<PositionedTiles> {
  TextStyle textStyle = TextStyle(fontSize: 16, fontWeight: FontWeight.bold,);
  var appBar= AppBar(title: Text('Exo 6'),);
  double _currentSlideValue=4;
  List<TileWidget> tiles=createTileWidgets(4,0);
  int emptyIndex=0;

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: appBar,
      body: ListView(children: [
        Container(
          height: ((MediaQuery.of(context).size.height)*0.7),
          child:GridView.builder(
            padding: const EdgeInsets.all(10),
            itemCount: _currentSlideValue.toInt()*_currentSlideValue.toInt(),
            gridDelegate:SliverGridDelegateWithFixedCrossAxisCount(
              crossAxisCount: _currentSlideValue.toInt(),
              crossAxisSpacing: 5.0,
              mainAxisSpacing: 5.0,
            ),
            itemBuilder: (BuildContext context, int index) {
              return Container(
                decoration: myBoxDecoration(tiles[index].isUsable),
                child:InkWell(
                  child: tiles[index],
                  onTap: (){
                    swapTiles(index,emptyIndex);
                  },
                ),
              );
            },
          ),
        ),
        Row(children: <Widget>[
              Container(width: 90,child:Text('Taille:', style: textStyle,textAlign: TextAlign.center,),padding: EdgeInsets.fromLTRB(8, 0, 0, 0),),
              Container(height:((MediaQuery.of(context).size.height)/7)-appBar.preferredSize.height,width: MediaQuery.of(context).size.width-90,child: Slider(
                  value: _currentSlideValue,
                  min: 2, max: 7, divisions: 5,
                  label: _currentSlideValue.round().toString(),
                  onChanged: (double value) {
                    setState(() {
                      _currentSlideValue = value;
                      tiles=createTileWidgets(_currentSlideValue,emptyIndex);
                      //initialSize=_currentSlideValue;
                    });
                  }
               ),
              ),
            ]
        ),
      ],
      )
    );
  }

  swapTiles(int current, int empty) {
    setState(() {
      //RESET bordures des cases apres deplacements
      for(int i=0;i<tiles.length;i++){
        if(i!=empty ) tiles[i].isUsable=false;
      }

      if(current==empty+1 && current%_currentSlideValue!=0){
        emptyIndex=current;
        tiles.insert(empty, tiles.removeAt(current));
       // tiles[empty].isUsable=true;
       // tiles[empty+2].isUsable=true;
      //  tiles[empty+_currentSlideValue.toInt()+1].isUsable=true;

      }else if(current==empty-1  && empty%_currentSlideValue!=0){
        emptyIndex=current;
        tiles.insert(empty, tiles.removeAt(current));
      }
      else if(current==empty+_currentSlideValue){
        emptyIndex=current;
        tiles.insert(current, tiles.removeAt(empty));
        tiles.insert(empty, tiles.removeAt(current-1));
      }else if(current==empty-_currentSlideValue){
        emptyIndex=current;
        tiles.insert(current, tiles.removeAt(empty));
        tiles.insert(empty, tiles.removeAt(current+1));
      }else print("Can't move this");

      if(emptyIndex>=_currentSlideValue) tiles[emptyIndex-_currentSlideValue.toInt()].isUsable=true;
      if(emptyIndex<(_currentSlideValue*(_currentSlideValue-1))) tiles[emptyIndex+_currentSlideValue.toInt()].isUsable=true;
      if((emptyIndex+1)%_currentSlideValue!=0)tiles[emptyIndex+1].isUsable=true;
      if(emptyIndex%_currentSlideValue!=0)tiles[emptyIndex-1].isUsable=true;
    });
  }

}